////////////////////////////////////////////////////////////////////////////////
// MAIN.C - CLI: Command Line Interface (works with 957,857,950 and 850)
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "icon.h"

char VersionPtr[] = "CLI";
int AppStackSize = 3000;

char buf[64];
LcdConfig LcdCfg;

void puts(char *s)
{
  LcdPutStringXY(0,LcdCfg.height-8,s,-1,0);
  LcdScroll(8);
}

int sstr(char *s,int c)
{
  char *s2=s;
  while(*s) if(*s++ == c) return (s-s2);
  return 0;
}

void info(void)
{
  FileSysInfoType FSI;
  DbFileSysInfo(&FSI);
  RimSprintf(buf,63,"Memory free: %d bytes",RimMemoryRemaining());
  puts(buf);
  RimSprintf(buf,63,"Flash free: %d bytes",DbFreeSpace());
  puts(buf);
  RimSprintf(buf,63,"Handles left: %d",DbFreeRec());
  puts(buf);
  RimSprintf(buf,63,"Data start address: %d",FSI.Disk);
  puts(buf);
  RimSprintf(buf,63,"Flash mem blocks #: %d",FSI.NumOfBlock);
  puts(buf);
  RimSprintf(buf,63,"Flash mem block size: %d",FSI.BlockSize);
  puts(buf);
  RimSprintf(buf,63,"Cleanups since turned on: %d",FSI.NumOfClean);
  puts(buf);
}

void CLIThread(void)
{
  MESSAGE msg; int i; unsigned char x;
  char KB[64]; char NC=0; char c;
  HandleType H; FileType F1, F2;
  PID pid; pid.Name = VersionPtr;
  pid.EnableForeground = TRUE; pid.Icon = NULL;
  RimSetPID(&pid); LcdGetConfig(&LcdCfg); KB[NC]=0;
  RimRequestForeground(RimGetCurrentTaskID());
  puts("CLI: Command Line Interface");puts("");

  for (;;) // Start message loop
  {
    LcdPutStringXY(0,LcdCfg.height-8,">",-1,0); // Prompt
    LcdPutStringXY(-1,-1,KB,-1,0);              // Command line
    LcdPutStringXY(-1,-1," ",1,TEXT_INVERT);    // Cursor
    LcdClearToEndOfLine();                      // Erase incase shorter
    RimGetMessage(&msg);

    switch (msg.Device)
    {
      case DEVICE_KEYPAD:
        switch (msg.Event)
        {
          case KEY_DOWN:
          {
            switch (msg.SubMsg)
            {
              case KEY_ESCAPE: // ESC key exits
                RibbonShowRibbon();
                RimTerminateThread();
                break;
              case KEY_ENTER:  // Command check
                LcdPutStringXY(0,LcdCfg.height-8,">",-1,0);
                LcdPutStringXY(-1,-1,KB,-1,0);
                LcdClearToEndOfLine(); LcdScroll(8);
                switch(*KB)
                {
                  case 'i': info(); break; // INFO
                  case 'r': DbDelete(&KB[3]); break; // RM
                  case 'h': puts("ls  info  help  secure");   // HELP
                    puts("rm <file>  cp <src> <des>"); break;
                  case 's': DbSecure();puts("SECURED");break; // SECURE
                  case 'l': H=DbFindNext(-1,"*"); // LS
                    while(H>=0) {
                      RimSprintf(buf,63,"%s[%d]",DbName(H),DbSize(H,TRUE));
                      puts(buf); H=DbFindNext(H,"*");
                    }
                    break;
                  case 'c': c=sstr(&KB[3],32); // CP
                    if (c) {
                      KB[c+2]=0; // Separate strings
                      H=DbGetHandle(&KB[3+c]); F2=DbFileOpen(H,256);
                      H=DbGetHandle(&KB[3]);   F1=DbFileOpen(H,256);
                      for(i=0; i<DbSize(H,TRUE); i++) {
                      DbFileRead(F1,&x,1);  DbFileWrite(F2,&x,1); }
                      DbFileClose(F1,TRUE); DbFileClose(F2,TRUE);
                    }else puts("File Not Found!");
                  break;
                  default: puts("Syntax Error!");
                }
                NC=0; KB[NC]=0;
                break;
              case KEY_BACKSPACE:
                if(NC>0) {NC--; KB[NC]=0;}
                break;
              default:
                KB[NC++]=msg.SubMsg;
                KB[NC]=0;
            }
            break;
          }  // End case
        }    // End switch
    }        // End switch
  }          // End message loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr,&IconBMP,0,40,CLIThread,AppStackSize);
}
